## How to install

* Install AutoMoDe (with all dependencies)

* Download the repository

* Create a virtual environment (optional)

* Install mpi4py, graphviz, numpy and scipy

## General Remarks ##

We are always talking about AutoMoDe as if it is a single program, but it's not.
AutoMoDe is our implementation of the modules and it uses the argos3 simulator to test the performance.

ArGoS3-beta48 is the version of the robot simulator that we use.
The ARGoS3-epuck v48 is a plugin for argos that implements the epuck robot that we are using for our experiments.
Together these two programs form our simulating environment that we use to perform our simulations.
The focus of them is to provide us with the performace quickly but accurately (that is as close to reality as we can get).

The repositories of AutoMoDe-FSM and AutoMoDe-BT contain the modules and the control architecture for AutoMoDe.
They make use of the epuck-DAO (an abstraction layer that gives us limited but focussed access to the sensors and actuators of the epuck) and the loopfunctions (implementations that set up and control the missions; think of what we would need to do in a real experiment: setting up the robots, counting the final score, ...). 
Together these implementations form what we usually refer to as AutoMoDe (and it is in fact missing the automatic design).

When we call the AutoMoDe executable we usually have to supply three arguments: seed, mission and controller.
The seed is simply the seed for the RNG and if it is kept the same (together with the other arguments) then the result is the same as well.
The mission (or scenario) is a XML file (with either .xml or .argos ending) that describes the layout for the arena, the number, type and controller of robots and the loopfunction that performs (supervises) the experiment.
The controller finally is a special string that is translated from AutoMoDe into an actual instance of control software that can run either in simulation or reality.
This string is usually generated by hand (for testing), F-Race (Vanilla) or I-Race (Chocolate and Maple).
We are now trying to generate this string using local search.
Once AutoMoDe has all of these arguments, it starts argos3 with the setup described in the mission file and all robots get the controller.
Argos3 then runs the simulation and the loop function (from the mission file) prints the final score.
This final score is then read by whatever optimization implementation is used.

The local search can be invoked by calling the file "automode_localsearch.py".
It operates on representations of the instances of control software.
It has no knowledge about the meaning/implementation of the individual modules and is working "blindly" on the design of the control software
("Blindly" in the sense that the local search has no insight in the workings of the modules, the mission or anything else. That is to say that we can simply import a different python file with modules or set a different mission and the local search will behave the same).
However the local search needs some information on how good or bad an instance of control software is.
For the evaluation it passes the control software (or more correctly a representation thereof) to our implementation of the AutoMoDe executable and that executable returns a single value (the performance measure/objective function).
According to a number of these values, the local search decides whether to keep or discard the current solution.

## Running a local search

The entry point for this program, is the file "automode_localsearch.py" in the directory "automode_localsearch".
Running "python3 automode_localsearch.py --help" will output a helpful information about how to use this program.

The program provides three subcommands that can be chosen: run, local, submit
The subcommand "run" will run a single run of the local search.
It expects the some parameters supplied via the commandline.
Run "python3 automode_localsearch.py run --help" for more information about these parameters.
The subcommands "local" and "submit" run a whole experiment suite.
They expect a single experiment file (supplied with the flag "-e") that describes all experiments.
The experiment files used for this paper, are available in the folder "experiments".
The files are named after the local search starting condition and their budget.
For example "python3 automode_localsearch.py local -e ../experiments/minimal50k.txt" will start 10 local search runs for every control architecture (BT or FSM) and every mission.
The difference between "local" and "submit" is, that "local" will attempt to run the program locally, while "submit" will submit the program to the scheduling system of a computing cluster first.
